/****************************************************************************
 * (c) 2019 softwareentwicklung-als-prozess.de
 * Funktionalitaet:
 * Frequenz messen testen und ausgeben mit einem ATMega328 - serieller Parser
 ***************************************************************************/
// Includes -----------------------------------------------------------------
#include <arduino.h>
#include <Wire.h>
#include "pwmctrl.h"
#include "gpio.h"
#include "testsuite.h"

// Defines -------------------------------------------------------------------

// Variablen -----------------------------------------------------------------

// interne Prototypen --------------------------------------------------------
static void mathPulseShowInfo(void);
static void error(int no);

// Funktionen ----------------------------------------------------------------
void spars_Init(void)
{
  //Hilfe text
  Serial.println(F("\nTST: TST: = Meldung vom Testsystem"));
  Serial.println(F("TST: DUT: = Meldung vom *Device under Test*"));
  Serial.println(F("TST: "));
  Serial.println(F("TST: \nTestsystem - Kommandos fuer Frequenz Messen:"));
  Serial.println(F("TST: -------------------------------------------"));
  Serial.println(F("TST: Frequenz nur im Bereich von 1 bis 1000000Hz"));
  Serial.println(F("TST: Puls / Pause nur im Bereich von 0 bis 100"));
  Serial.println(F("TST: "));
  Serial.println(F("TST: Eingabe: F1000 - Frequenz setzen auf 1000Hz"));
  Serial.println(F("TST: Eingabe: W50   - Pulse/Pause setzen auf 50%"));
  Serial.println(F("TST: Eingabe: R     - Reset des *Device under Test*"));
  Serial.println(F("TST: Eingabe: T     - Testsuite starten fuer *Device under Test*"));
  Serial.println(F("TST: -------------------------------------------"));
}

void spars_Get(void) // Test Kommandos einlesen
{
  while (Serial.available()) 
  {
    String read = Serial.readStringUntil('\n'); // Steuerkommando abholen
    
    if (read.substring(0, 1) == "F") // Frequenz einstellen
    {
      long frq = read.substring(1, read.length()).toInt();
      if (frq < (long)0 || frq > (long)1000000) 
      { 
        error((int)2);
        break; 
      }
      pwm_SetzeFrequenz(frq);
      mathPulseShowInfo();
    }
    if (read.substring(0, 1) == "W") // Pulsbreite einstellen
    {
      long width = read.substring(1, read.length()).toInt();
      if (width < (long)0 || width > (long)100) 
      {  
        error((int)1); 
        break; 
      }
      pwm_SetzePulsBreite(width);
      mathPulseShowInfo();
    }
    if (read.substring(0, 1) == "R") // Reset ausfuehren
    {
      gpio_Reset();
      pwm_SetzeFrequenz(1l);
      mathPulseShowInfo();
      Serial.println(F("TST: Reset ausgefuehrt\n"));
    }
    if (read.substring(0, 1) == "T") // Testsuite ausfuehren
    {
      tst_All();
    }
  }
}
    
static void mathPulseShowInfo(void)
{
  Serial.print(F("\nTST: Ausgabe auf Pin : "));
  Serial.println(pwm_GetOutputPin());

  Serial.print(F("TST: Frequenz: "));
  Serial.print(pwm_GetFrequenz());
  Serial.println(F("Hz"));

  Serial.print(F("TST: Pulsbreite: "));
  Serial.print(pwm_GetPulsBreite());
  Serial.println(F("%"));

  float pulseHighUs = ((1000000.0f / pwm_GetFrequenz()) * pwm_GetPulsBreite()) / 100.0f;
  Serial.print(F("TST: > Pulsbreite: +"));
  Serial.print(pulseHighUs);
  Serial.println(F(" us"));

  float pulseLowUs = ((1000000.0f / pwm_GetFrequenz()) * (100.0f - pwm_GetPulsBreite())) / 100.0f;
  Serial.print(F("TST: > Pulsbreite: -"));
  Serial.print(pulseLowUs);
  Serial.println(F(" us\n"));
}

static void error(int no) 
{
  Serial.print(F("TST: Fehler: "));
  switch (no)
  {
  case 1:
      Serial.println(F("TST: Puls / Pause nur im Bereich von 0 bis 100\n"));
      break;
  case 2:
      Serial.println(F("TST: Frequenz nur im Bereich von 1 bis 1000Hz\n"));
      break;
  }
}
