/****************************************************************************
 * (c) 2019 softwareentwicklung-als-prozess.de
 * Funktionalitaet:
 * Frequenz messen - Testsuite mit Testfaellen
 ***************************************************************************/
// Includes -----------------------------------------------------------------
#include <arduino.h>
#include "testsuite.h"
#include "twictrl.h"
#include "pwmctrl.h"
#include "gpio.h"

// Defines -------------------------------------------------------------------
// Testfaelle 1 bis 6
#define HZ30 (long)30
#define HZ31 (long)31
#define HZ60 (long)60
#define HZ90 (long)90
#define HZ99 (long)99
#define HZ100 (long)100

// Variablen -----------------------------------------------------------------
static char tsttxt[TXTLEN];
static byte frq = (byte)0;

// interne Prototypen --------------------------------------------------------
static byte FrequenzEinlesen(void);
static void HoleNachricht(void);
static void cleartext(void);

// Funktionen ----------------------------------------------------------------
void tst_All(void)
{
  tst_T1();
  tst_T2();
  tst_T3();
  tst_T4();
  tst_T5();
  tst_T6();
  tst_T7();
}

void tst_T1(void)
{
  Serial.println(F("TST: Testfall 1 - 30Hz"));
  pwm_SetzeFrequenz(HZ30);
  delay(2000);
  HoleNachricht();
  if (0 == strcmp(tsttxt, "ausser Bereich! "))
  {
    Serial.println(F("TST: OK - Testfall 1 bestanden"));
  }
  else
  {
    Serial.println(F("TST: NOK - Testfall 1 *nicht* bestanden"));    
  }
  cleartext();
}

void tst_T2(void)
{   
  Serial.println(F("TST: Testfall 2 - 31Hz"));
  pwm_SetzeFrequenz(HZ31);
  delay(2000);
  HoleNachricht();
  frq = FrequenzEinlesen();
  if ((frq >= (byte)30) || (frq <= 32))
  {
    Serial.println(F("TST: OK - Testfall 2 bestanden"));
  }
  else
  {
    Serial.println(F("TST: NOK - Testfall 2 *nicht* bestanden"));    
  }
  cleartext();
}

void tst_T3(void)
{
  Serial.println(F("TST: Testfall 3 - 60Hz"));
  pwm_SetzeFrequenz(HZ60);
  delay(2000);
  HoleNachricht();
  frq = FrequenzEinlesen();
  if ((frq >= (byte)59) || (frq <= (byte)61))
  {
    Serial.println(F("TST: OK - Testfall 3 bestanden"));
  }
  else
  {
    Serial.println(F("TST: NOK - Testfall 3 *nicht* bestanden"));    
  }
  cleartext();
}

void tst_T4(void)
{
  Serial.println(F("TST: Testfall 4 - 90Hz"));
  pwm_SetzeFrequenz(HZ90);
  delay(2000);
  HoleNachricht();
  frq = FrequenzEinlesen();
  if ((frq >= (byte)89) || (frq < (byte)91))
  {
    Serial.println(F("TST: OK - Testfall 4 bestanden"));
  }
  else
  {
    Serial.println(F("TST: NOK - Testfall 4 *nicht* bestanden"));    
  }
  cleartext();
}

void tst_T5(void)
{
  Serial.println(F("TST: Testfall 5 - 99Hz"));
  pwm_SetzeFrequenz(HZ99);
  delay(2000);
  HoleNachricht();
  if (0 == strcmp(tsttxt, "ausser Bereich! "))
  {
    Serial.println(F("TST: OK - Testfall 5 *nicht* bestanden"));
  }
  else
  {
    frq = FrequenzEinlesen();
    if ((frq >= (byte)97) || (frq < (byte)100))
    {
      Serial.println(F("TST: OK - Testfall 5 bestanden"));
    }
    else
    {
      Serial.println(F("TST: NOK - Testfall 5 *nicht* bestanden"));    
    }
  }
  cleartext();
}

void tst_T6(void)
{
  Serial.println(F("TST: Testfall 6 - 100Hz"));
  pwm_SetzeFrequenz(HZ100);
  delay(2000);
  HoleNachricht();
  if (0 == strcmp(tsttxt, "ausser Bereich! "))
  {
    Serial.println(F("TST: OK - Testfall 6 bestanden"));
  }
  else
  {
    Serial.println(F("TST: NOK - Testfall 6 *nicht* bestanden"));    
  }
  cleartext();
}

void tst_T7(void)
{
  gpio_Reset();
  Serial.println(F("TST: Testfall 7 - Reset"));
  pwm_SetzeFrequenz(HZ100);
  delay(3000);
  HoleNachricht();
  if (0 == strcmp(tsttxt, "Starte Frequenz messen"))
  {
    Serial.println(F("TST: OK - Testfall 7 bestanden"));
  }
  else
  {
    Serial.println(F("TST: NOK - Testfall 7 *nicht* bestanden"));    
  }
  cleartext();
}

// Frequenz einlesen
static byte FrequenzEinlesen(void)
{
  frq = (byte)0;
  return(((tsttxt[7] - 0x30) * 10) + (tsttxt[8] - 0x30));
}

// Nachricht abholen
static void HoleNachricht(void)
{
  if ((bool)false == twi_CheckMessage())
  {
    while((bool)false == twi_CheckMessage())
    {
    }
  }
  twi_CopyMessage(tsttxt);                // Text einlesen
  Serial.print("TST: ");
  Serial.println(tsttxt); 
}

// Textausgabefeld initialisieren
static void cleartext(void)
{
  int i = 0;

  for(i = 0; i < TXTLEN; i++)
  {
    tsttxt[i] = ENDE;
  }
}
